/*
* DotProduct - Tristan Aubrey-Jones 2014
* Calculates the dot product of two randomly generated vectors stored as distributed lists.
*/
#include <iostream>
#include <vector>
#include <cstdlib>
#include <mpi.h>

#define ASz 200000000
#define BSz 100000000
#define RAND_DOUBLE ((double)rand()/RAND_MAX)

using namespace std;

int main(int argc, char** argv) {
  MPI::Init(argc, argv);
  int rank = MPI::COMM_WORLD.Get_rank();
  int N = MPI::COMM_WORLD.Get_size();
  double startT = MPI::Wtime();

  // generate local data
  int An = (ASz/N) + (rank < (ASz%N) ? 1 : 0);
  int Bn = (BSz/N) + (rank < (BSz%N) ? 1 : 0); 
  vector<double> A(An), B(Bn);
  for (int i = 0; i < An; i++) A.push_back(RAND_DOUBLE);
  for (int i = 0; i < Bn; i++) B.push_back(RAND_DOUBLE);

  // compute local dot product
  double sum = 0.0;
  typedef vector<double>::iterator it;
  for (it ai = A.begin(), bi = B.begin();
       ai != A.end() && bi != B.end();
       ai++, bi++) {
    double v = (*ai) * (*bi);
    sum += v;
  }

  // all reduce to get sum
  double globalSum = 0.0;
  MPI::COMM_WORLD.Reduce(&sum, &globalSum, 1, MPI::DOUBLE, MPI::SUM, 0);

  // display
  if (rank == 0) {
    double dur = MPI::Wtime() - startT;
    std::cout << "dotprod\t" << ASz << "\t" << BSz << "\t" << N << "\t" << dur << "\n";
  }

  MPI::Finalize();
  return 0;
}
